// Filename: AllTest.ino
// Author: Akinori TSuji
//#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include "FastLED.h"
#include "RTClib.h"
#include "SparkFunBME280.h"

//---FastLED(WS2811) D12
const int DATA_PIN = 12;
#define NUM_LEDS  1
CRGB leds[NUM_LEDS];

//---BME280
BME280  bme280;

//---DS1307
RTC_DS1307 rtc;
char daysOfTheWeek[7][12] =
{"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};

//---LCD1602 (0x27)
LiquidCrystal_I2C lcd(0x27, 16, 2);

// Custom character
uint8_t temp[8] = {
  B01110,
  B01010,
  B01010,
  B01010,
  B01110,
  B01110,
  B01110,
  B00100
};
uint8_t humid[8] = {0x4, 0x4, 0xa, 0xa, 0x11, 0x11, 0x11, 0xe};
uint8_t heart1[8] = {0x0, 0xa, 0x1f, 0x1f, 0xe, 0xe, 0x4};
uint8_t heart2[8] = {0x0, 0x0, 0xa, 0x1f, 0xe, 0x4, 0x0};
//uint8_t heart[8] = {0x0, 0xa, 0x1f, 0x1f, 0xe, 0x4, 0x0};
//uint8_t clock[8] = {0x0, 0xe, 0x15, 0x17, 0x11, 0xe, 0x0};
uint8_t pressure[8] = {0x04, 0xe, 0xe, 0x11, 0x11, 0x1f, 0x1f};
uint8_t bar1[8] = {0x0, 0x0,  0x0,  0x0,  0x0,  0x1f, 0x1f};
uint8_t bar2[8] = {0x0, 0x0,  0x0,  0x1f, 0x1f, 0x1f, 0x1f};
uint8_t bar3[8] = {0x0, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f};

// Digital output
#define LED_PIN 13

void setup()   /*----( SETUP: RUNS ONCE )----*/
{
  //---LED (D13)
  pinMode(LED_PIN, OUTPUT);

  //---BME280
  bme280.settings.commInterface = I2C_MODE;
  bme280.settings.I2CAddress = 0x76;
  bme280.settings.runMode = 3; //Normal mode
  bme280.settings.tStandby = 0;
  bme280.settings.filter = 0;
  bme280.settings.tempOverSample = 1;
  bme280.settings.pressOverSample = 1;
  bme280.settings.humidOverSample = 1;
  bme280.begin();

  //---FastLED
  FastLED.addLeds<WS2811, DATA_PIN, RGB>(leds, NUM_LEDS);
  FastLED.setBrightness(16); // 0-255
  leds[0] = CRGB(0, 0, 0); // 消灯
  FastLED.show();

  //---DS1307
  if (!rtc.begin()) {
    //    Serial.println("Couldn't find RTC");
    //    while (1);
  }

  if (!rtc.isrunning()) {
    //    Serial.println("RTC is NOT running!");
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // January 21, 2014 at 3am you would call:
    // rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));
  }

  //---Serial
  Serial.begin(9600);

  //---LCD1602
  lcd.init(); // initialize the lcd for 16 chars 2 lines, turn on backlight
  lcd.backlight();
  lcd.createChar(0, temp);
  lcd.createChar(1, humid);
  lcd.createChar(2, heart1);
  lcd.createChar(3, heart2);
  lcd.createChar(4, pressure);
  lcd.createChar(5, bar1);
  lcd.createChar(6, bar2);
  lcd.createChar(7, bar3);
  lcd.clear();
  lcd.home();
}

int toggle = 0;
int time_cur = 0, time_last = 0;
int h = 0;
float coeff = 3.0;

void loop() {
  // 1秒毎に表示更新
  time_cur = millis();
  if (time_cur - time_last > 1000) {
    time_last = millis();

    //---FastLED
    leds[0] = CHSV(h, 255, 255);
    FastLED.show();
    h += 10;
    if (h > 255) h = 0;

    //--- temperature, humidity, pressure
    /*
      高さ760mm，底面積 1m^2 の水銀柱による力1.01325×105 N．
      単位面積は1m^2，したがって
      1 atm = 1.01325×10^5 N/m2
      N/m2 は気圧の単位 Pa
      1 atm = 1.01325×10^5 Pa
      さらに，単位接頭記号の h = 10^2 を使うと
      1 atm = 1.01325×10^3 hPa = 1013.25 hPa
    */
    lcd.setCursor(0, 0);
    lcd.write(0); // temprature
    lcd.print(bme280.readTempC(), 1);
    lcd.print(" ");
    lcd.write(1); // humidity
    lcd.print(bme280.readFloatHumidity(), 0);
    lcd.print("%");
    lcd.print(" ");
    lcd.write(4); // pressure
    lcd.print(bme280.readFloatPressure() / 100.0, 0);

    // heart beat
    lcd.setCursor(0, 1);
    if (toggle == 1) {
      lcd.write(2);
      digitalWrite(LED_PIN, 1);
      toggle = 0;
    } else {
      lcd.write(3);
      digitalWrite(LED_PIN, 0);
      toggle = 1;
    }

    //--- time
    DateTime now = rtc.now();
    lcd.setCursor(1, 1);
    lcd.print(now.year()-2000);
    lcd.print('/');
    lcdprint_zero(now.month());
    lcd.print('/');
    lcdprint_zero(now.day());
    lcd.print(' ');
    lcdprint_zero(now.hour());
    lcd.print(':');
    lcdprint_zero(now.minute());

    //---lux
    int data = analogRead(A0);
    float vo = data * 3.3 / 1024.0;
    float lux = coeff * 222.0 * vo; // 0lux - 2195lux
    illum(lux);
    Serial.println(lux);
  }
}

void illum(float lux) {
  lcd.setCursor(15, 1);
  int pos = map(lux, 0.0, 800.0, 0, 3);
  switch (pos) {
    case 0:
      lcd.print(" ");
      break;
    case 1:
      lcd.write(5);
      break;
    case 2:
      lcd.write(6);
      break;
    case 3:
      lcd.write(7);
      break;
    default:
      break;
  }
}

void lcdprint_zero(int n) {
  if (n >= 0 && n < 10) {
    lcd.write('0');
  }
  lcd.print(n);
}

